#include "JStrSplitObj.h"
#include "JInteger.h"
#include "JIntegerData.h"
#include "JStringData.h"
#include "JIntegerProperty.h"
#include "JIntegerListProperty.h"
#include "JStringProperty.h"
#include "JLinkObj.h"
#include "JBytesData.h"

/**
  
	This module listens for String inputs, looking for a specific piece of text.
	After it finds that text, it sends it out, then sends out fixed size string
	segments.  The "startStr" and "segmentLength" are properties of this module.

	The corresponding "stopStr" can be detected by a StrArea module, which looks
	for a "startStr" and a "stopStr".  When the StrArea does not have a "startStr",
	it matches anything, and only looks for the "stopStr".

  */

char* theJStrSplitObj = JStrSplitObj().Register();


void JStrSplitObj::writeContent(JOutputStream& os) {
  JAddObj::writeContent(os);
  putObject(os, "startStr", startStr);
  putInteger(os, "segmentLength", segmentLength);
}

void JStrSplitObj::readContent(JDictionary& dict) {
  JAddObj::readContent(dict);
  JObject* obj_startStr = getObject( dict, "startStr");
  if ( obj_startStr ) startStr = *(JString*)obj_startStr;
  segmentLength = getInteger(dict, "segmentLength");
}

const char* JStrSplitObj::className() const { return "JStrSplitObj";}


JObject* JStrSplitObj::clone() const { return new JStrSplitObj(*this);}


// class constructor
JStrSplitObj::JStrSplitObj() {
  igm = 1 + 1 + 1;
  ogm = 1;
  enabled = 0;
  reset = 0;
  broadcasting = false;
}


void JStrSplitObj::draw(JGraphics g, int x, int y, int w, int h) {
  JRect rect(x, y, w, h);
  if ( startStr.length() > 0 ) {
	  drawText( g, startStr, rect );
  } else {
	drawText(g, JString("StrSplit"), rect);
  }
}


boolean JStrSplitObj::inputAllowed(int n)
{ if ( n==2) return( true ); 
return !inputSet(n).last();}
JString JStrSplitObj::inputTag(int n) {
  char* tag[] = {
    "text",
    "enabled",
    "reset" };
  return( tag[n] );
}


int JStrSplitObj::outputType( int n ) {
  if ( n == 0 ) return JStringData::id;
  return JIntegerData::id;
}
int JStrSplitObj::inputType( int n ) {
  if ( n == 0 ) return JStringData::id;
  return JIntegerData::id;
}
JString JStrSplitObj::outputTag(int n) {
  char* tag[] = {
    "string_segment" };
  return( tag[n] );
}


JFRect JStrSplitObj::inputArea(int n) {
  if ( n < ( 1 )) {
    return leftArea( n, 0, 1, 0.1, 0.8 );
  } else {
    return topArea( n, 1, 1 + 1, 0.25 );
  }
}


JFRect JStrSplitObj::outputArea(int n) {
  return rightArea(n, 0, 1);
}


int JStrSplitObj::inputFace(int n) {
  if ( n < 1 ) {
    return LEFT;
  } else {
    return TOP;
  }
}


int JStrSplitObj::outputFace(int n) {
  return RIGHT;
}


JFPoint JStrSplitObj::inputPoint(int n, JLinkObj& link) {
  if (n<1) return leftPoint(n, link, 0, 1, 0.1, 0.8);
  return topPoint(n, link, 1, 1 + 1);
}


JFPoint JStrSplitObj::outputPoint(int n, JLinkObj& link) {
  return rightPoint(n, link, 0, 1);
}


void JStrSplitObj::access(int n, JLinkObj& link, const JDataType& data) {
 // if ( n == 0 ) {
    JSTRING(data) = string_segment;
  //}
}

#include <string.h>
void JStrSplitObj::processText( char* _text ) {
	if (!broadcasting && ( startStr.length() > 0 )) {
		char* foundLocation = strstr( _text, startStr );
		if ( foundLocation != NULL ) {
			string_segment = startStr;
			//JComponent::debug( "debug initial segment" );
			broadcast( 0 );
			broadcasting = true;
			saveBuffer[0] = 0;
			//JComponent::debug( "broadcast rest" );
			broadcastText( _text + strlen( startStr ));
		}
	} else if ( broadcasting ) {
		char buf[500];
		buf[0] = 0;
		if ( strlen( saveBuffer ) > 0 ) {
			strcpy( buf, saveBuffer );
		}
		strcat( buf, _text );
		broadcastText( &buf[0] );
	}
}

#include <malloc.h>
#include <string.h>
#include <stdio.h>

char* newstrcpy( char* s ) {
	char* result = (char*)malloc( strlen( s ) + 1 );
	strcpy( result, s );
	return( result );
}

void JStrSplitObj::broadcastText( char* text ) {
	char result[500];
	sprintf( result, "broadcastText %s", text );
	//JComponent::debug( newstrcpy( result ));
	while ( strlen( text ) > segmentLength ) {
		strncpy( &result[0], text, segmentLength );
		result[segmentLength] = 0;
		//char* temp = newstrcpy( result );
		string_segment = JString( &result[0], 2);
		broadcast( 0 );
		//JComponent::debug( string_segment );
		//free( temp );
		text += segmentLength;
	}
	if ( strlen( text ) > 0 ) {
		strcpy( saveBuffer, text );
	}
}
		
void JStrSplitObj::engine(int n, JLinkObj& link) {
  switch (n) {
  case TEXT:
    link.access( JStringData( text ));
	if ( enabled ) {
		processText( text );
	}
    break;
  case ENABLED:
    link.access(JIntegerData(enabled));
    break;
  case RESET:
    link.access(JIntegerData(reset));
	if ( reset ) {
		broadcasting = false;
	}
    break;
  default:
    break;
  }
}


JArray JStrSplitObj::getProperties() {
  JArray properties;
  properties.append(JStringProperty("startStr", startStr));
  properties.append(JIntegerProperty("segmentLength", segmentLength, 0, 500));
  return properties;
}


boolean JStrSplitObj::updateProperty(JProperty& prop) {
  if (prop.getName() == JString("startStr")) {
    startStr = ((JStringProperty*)&prop)->value;
    return true;
  } else if (prop.getName() == JString("segmentLength")) {
    segmentLength = ((JIntegerProperty*)&prop)->value;
    return true;
  }
  return false;
}


